@@ -119,9 +119,10 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
         int *input_fds_count, int *output_fds,
         int *output_fds_count, int is_client)
 {
-    int opt;
+    int opt, command = 0;
+    int longindex;
     int i, ok;
-    int mode_list_keys = 0, mode_verify = 0;
+    bool userid_args = false, mode_verify = false;
 
     *input_fds_count = 0;
     *output_fds_count = 0;
@@ -132,9 +133,9 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
     output_fds[(*output_fds_count)++] = 2;	//stderr
 
     /* getopt will filter out not allowed options */
-    while ((opt =
+    while (longindex = -1, (opt =
                 getopt_long(argc, untrusted_argv, gpg_short_options,
-                    gpg_long_options, NULL)) != -1) {
+                    gpg_long_options, &longindex)) != -1) {
         if (opt == '?' || opt == ':') {
             /* forbidden/missing option - abort execution */
             //error message already printed by getopt
@@ -150,14 +151,28 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
             i++;
         }
         if (!ok) {
-            fprintf(stderr, "Forbidden option: %s\n",
-                    untrusted_argv[optind - 1]);
+            if (longindex != -1)
+                fprintf(stderr, "Forbidden option: --%s\n",
+                        gpg_long_options[longindex].name);
+            else
+                fprintf(stderr, "Forbidden option: -%c\n", opt);
             exit(1);
         }
-        if (opt == 'k' || opt == 'K' || opt == opt_export || opt == opt_export_ssh) {
-            // --list-keys and --export can have multiple key IDs as arguments
-            mode_list_keys = 1;
-        } else if (opt == opt_status_fd) {
+        i = 0;
+        while (gpg_commands[i].opt) {
+            if (gpg_commands[i].opt == opt) {
+                if (command && userid_args != gpg_commands[i].userid_args) {
+                    /* gpg gives similarly vague error message */
+                    fprintf(stderr, "conflicting commands\n");
+                    exit(1);
+                }
+                command = opt;
+                userid_args = gpg_commands[i].userid_args;
+                break;
+            }
+            i++;
+        }
+        if (opt == opt_status_fd) {
             add_arg_to_fd_list(output_fds, output_fds_count);
         } else if (opt == opt_logger_fd) {
             add_arg_to_fd_list(output_fds, output_fds_count);
@@ -180,7 +195,7 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
         }
 
     }
-    if (mode_list_keys) {
+    if (userid_args) {
         // all the arguments are key IDs/user IDs, so do not try to handle them
         // as input files
         optind = argc;
