@@ -89,34 +89,37 @@ class TC_00_Direct(SplitGPGBase):
 
     def test_010_list_keys(self):
         cmd = 'qubes-gpg-client-wrapper --list-keys'
-        p = self.frontend.run(cmd, passio_popen=True)
-        (keys, _) = p.communicate()
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        (keys, stderr) = p.communicate()
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, stderr.decode()))
         self.assertIn("Qubes test", keys.decode())
 
     def test_020_export_secret_key_deny(self):
         # TODO check if backend really deny such operation, here it is denied
         # by the frontend
         cmd = 'qubes-gpg-client-wrapper -a --export-secret-keys user@localhost'
-        p = self.frontend.run(cmd, passio_popen=True)
-        keys, _ = p.communicate()
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        keys, stderr = p.communicate()
         self.assertNotEquals(p.returncode, 0,
-            '{} succeeded unexpectedly'.format(cmd))
+            '{} succeeded unexpectedly: {}'.format(cmd, stderr.decode()))
         self.assertEquals(keys.decode(), '')
 
     def test_030_sign_verify(self):
         msg = "Test message"
         cmd = 'qubes-gpg-client-wrapper -a --sign'
-        p = self.frontend.run(cmd, passio_popen=True)
-        (signature, _) = p.communicate(msg.encode())
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        (signature, stderr) = p.communicate(msg.encode())
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, stderr.decode()))
         self.assertNotEquals('', signature.decode())
 
         # verify first through gpg-split
         cmd = 'qubes-gpg-client-wrapper'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         (decoded_msg, verification_result) = p.communicate(signature)
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), msg)
         self.assertIn('\ngpg: Good signature from', verification_result.decode())
 
@@ -143,14 +146,17 @@ class TC_00_Direct(SplitGPGBase):
         msg = "Test message"
         self.frontend.run('echo "{}" > message'.format(msg), wait=True)
         cmd = 'qubes-gpg-client-wrapper -a -b --sign message > signature.asc'
-        p = self.frontend.run(cmd, wait=True)
-        self.assertEquals(p, 0, '{} failed'.format(cmd))
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        stdout, stderr = p.communicate()
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, stderr.decode()))
 
         # verify through gpg-split
         cmd = 'qubes-gpg-client-wrapper --verify signature.asc message'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         decoded_msg, verification_result = p.communicate()
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), '')
         self.assertIn('\ngpg: Good signature from', verification_result.decode())
 
@@ -160,7 +166,7 @@ class TC_00_Direct(SplitGPGBase):
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         decoded_msg, verification_result = p.communicate()
         self.assertNotEquals(p.returncode, 0,
-            '{} unexpecedly succeeded'.format(cmd))
+            '{} unexpecedly succeeded: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), '')
         self.assertIn('\ngpg: BAD signature from', verification_result.decode())
 
@@ -208,8 +214,8 @@ Expire-Date: 0
         if 'whonix' in self.template:
             self.frontend.run("date -s -10min", user="root", wait=True)
         p = self.frontend.run('mkdir -p -m 0700 .gnupg; gpg2 --gen-key --batch',
-                passio_popen=True)
-        p.communicate('''
+                passio_popen=True, passio_stderr=True)
+        stdout, stderr = p.communicate('''
 Key-Type: RSA
 Key-Length: 1024
 Key-Usage: sign
@@ -222,7 +228,8 @@ Expire-Date: 0
 %no-protection
 %commit
         '''.encode())
-        assert p.returncode == 0, 'key generation failed'
+        assert p.returncode == 0, 'key generation failed: {}'.format(
+            stderr.decode())
         # see comment in setUp()
         if 'whonix' in self.template:
             self.frontend.run("date -s +10min", user="root", wait=True)
@@ -248,15 +255,17 @@ Expire-Date: 0
         """Test for --output option"""
         msg = "Test message"
         cmd = 'qubes-gpg-client-wrapper -a --sign --output /tmp/signed.asc'
-        p = self.frontend.run(cmd, passio_popen=True)
-        p.communicate(msg.encode())
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        stdout, stderr = p.communicate(msg.encode())
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, stderr.decode()))
 
         # verify first through gpg-split
         cmd = 'qubes-gpg-client-wrapper /tmp/signed.asc'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         decoded_msg, verification_result = p.communicate()
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), msg)
         self.assertIn('\ngpg: Good signature from', verification_result.decode())
 
@@ -267,7 +276,8 @@ Expire-Date: 0
               '/tmp/signed.asc'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         (stdout, stderr) = p.communicate(msg.encode())
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, stderr.decode()))
         self.assertTrue(all(x.startswith('[GNUPG:]') for x in
             stdout.decode().splitlines()), "Non-status output on stdout")
 
@@ -275,7 +285,8 @@ Expire-Date: 0
         cmd = 'qubes-gpg-client-wrapper /tmp/signed.asc'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         decoded_msg, verification_result = p.communicate()
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), msg)
         self.assertIn('\ngpg: Good signature from', verification_result.decode())
 
@@ -299,10 +310,78 @@ Expire-Date: 0
         cmd = 'qubes-gpg-client-wrapper /tmp/signed.asc'
         p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
         decoded_msg, verification_result = p.communicate()
-        self.assertEquals(p.returncode, 0, '{} failed'.format(cmd))
+        self.assertEquals(p.returncode, 0,
+            '{} failed: {}'.format(cmd, verification_result.decode()))
         self.assertEquals(decoded_msg.decode(), msg)
         self.assertIn('\ngpg: Good signature from', verification_result.decode())
 
+    def _check_if_options_takes_argument(self, prog, option, message_fmts):
+        """Check whether an option expect an argument or not.
+        The *prog* will be called with *option* and --garbage-1 --garbage-2.
+        Based on which one is rejected, it will deduce whether the option
+        requires an argument or not.
+
+        :param prog: program to test (gpg2, qubes-gpg-client)
+        :param option: option to test
+        :param message_fmt: error message format, about rejected option
+        """
+
+        # check if option requires an argument by seeing if --garbage-1 was
+        # interpreted as another option, or an argument
+        cmd = '{} {} --garbage-1 --garbage-2'.format(prog, option)
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        (stdout, stderr) = p.communicate()
+        stderr = stderr.decode()
+        self.assertNotEquals(p.returncode, 0,
+            cmd + ' should have failed: ' + stderr)
+        for message_fmt in message_fmts:
+            if message_fmt.format('--garbage-1') in stderr:
+                return False
+            if message_fmt.format('--garbage-2') in stderr:
+                return True
+            if message_fmt.format(option) in stderr:
+                return None
+        if 'invalid argument for option "{}"'.format(option) in stderr:
+            return True
+        if 'Invalid fd argument' in stderr:
+            return True
+        self.fail(
+            '{} {} have not complained about --garbage options: {}'.format(
+                'qubes-gpg-client' if 'qubes' in prog else 'gpg2',
+                option, stderr))
+
+    def test_080_option_parser(self):
+        """Check if split-gpg agrees with gpg about options parsing"""
+        cmd = 'gpg --dump-options'
+        p = self.frontend.run(cmd, passio_popen=True, passio_stderr=True)
+        (stdout, stderr) = p.communicate()
+        self.assertEquals(p.returncode, 0, '{} failed: {}'.format(cmd,
+            stderr.decode()))
+        all_options = stdout.decode().splitlines()
+        noarg_options = []
+        for opt in all_options:
+            if opt in ('--output', '--logger-fd', '--version'):
+                # those options are problematic for testing (different error
+                # messages, messes with logging) and are checked manually
+                continue
+            splitgpg_needs_arg = self._check_if_options_takes_argument(
+                'QUBES_GPG_DOMAIN={} qubes-gpg-client'.format(self.backend.name),
+                opt, ['unrecognized option \'{}\'',
+                      'option \'{}\' is ambiguous',
+                      'Forbidden option: {}'])
+            if splitgpg_needs_arg is None:
+                # option rejected
+                continue
+            gpg_needs_arg = self._check_if_options_takes_argument(
+                'gpg2', opt, ['invalid option "{}"'])
+            self.assertEquals(gpg_needs_arg, splitgpg_needs_arg,
+                'gpg and splitgpg disagrees on {} option: {}, {}'.format(
+                    opt, gpg_needs_arg, splitgpg_needs_arg))
+            if not gpg_needs_arg:
+                noarg_options.append(opt)
+        # TODO: Test if gpg agrees with split-gpg, whether positional
+        #  argument(s) are a path or user id. Somehow...
+
     # TODO:
     #  - encrypt/decrypt
     #  - large file (bigger than pipe/qrexec buffers)
